# Python imports
import os
from zLOG import LOG, INFO, DEBUG

# Skins to replace Plone's ones
SKINS = {
    'euralba_form_scripts' : 'Products/EuralbaFilePatch/skins/plone_form_scripts',
}

log_ok_message = '...Already correctly installed'

class EuralbaFilePatchInstaller:
    product_name = 'EuralbaFilePatch'
    
    def __init__(self, context, product_name=None):
        """EuralbaFilePatchInstaller initialization"""
        self.context = context
        self.messages = []
        self.portal = context.portal_url.getPortalObject()
        if not hasattr(self.portal, '_v_main_installer'):
            self.portal._v_main_installer = self
        if product_name is not None:
            self.product_name = product_name
        if self.product_name is None:
            raise ValueError('No product name given to installer')
    
    def isMainInstaller(self):
        if getattr(self.portal, '_v_main_installer', 1) is self:
            return 1
        return 0

    def finalize(self):
        if self.isMainInstaller():
            self._CMFFinalize()

    def _CMFFinalize(self):
        """Does all the things that only needs to be done once"""
        self.reindexCatalog()
        self.resetSkinCache()
        self.reindexSecurity()
        if hasattr(self.portal, '_v_main_installer'):
            delattr(self.portal, '_v_main_installer')
            
    #
    # Methods normally called only at the end of an install.
    # Typically reindexing methods and similar.
    #
    def reindexCatalog(self):
        # Reindex portal_catalog
        reindex_catalog = getattr(self.portal, '_v_reindex_catalog', 0)
        changed_indexes = getattr(self.portal, '_v_changed_indexes', [])
        if reindex_catalog or len(changed_indexes) > 1:
            self.log('Reindex Catalog')
            self.portal.portal_catalog.refreshCatalog(clear=1)
        elif changed_indexes:
            ct = self.portal.portal_catalog
            self.log('Reindex Catalog indexes %s' % ', '.join(changed_indexes))
            for name in changed_indexes:
                ct.reindexIndex(name, self.portal.REQUEST)
        setattr(self.portal, '_v_reindex_catalog', 0)
        setattr(self.portal, '_v_changed_indexes', [])

    def resetSkinCache(self):
        # Reset skins cache
        if getattr(self.portal, '_v_reset_skins', 0):
            self.log("Resetting skin cache")
            self.portal._v_skindata = None
            self.portal.setupCurrentSkin()
        setattr(self.portal, '_v_reset_skins', 0)

    def reindexSecurity(self):
        if getattr(self.portal, '_v_reindex_security', 0):
            self.log("Reindexing Security")
            self.portal.reindexObjectSecurity()
        setattr(self.portal, '_v_reindex_security', 0)

    #
    # Logging
    #
    def log(self, message):
        self.messages.append(message)
        LOG(self.product_name, INFO, message)

    def logOK(self):
        self.messages[-1] = self.messages[-1] + log_ok_message
        #self.log(log_ok_message)

    def flush(self):
        log = '\n'.join(self.messages)
        self.messages = []
        return log

    def logResult(self):
        if not self.isMainInstaller():
            return self.flush()
        # Wrap HTML around it if it's the main installer.
        return '''<html><head><title>%s</title></head>
            <body><pre>%s</pre></body></html>''' % (
            self.product_name, self.flush() )

    #
    # Other support methods
    #
    def portalHas(self, id):
        return id in self.portal.objectIds()

    def getTool(self, id):
        return getToolByName(self.portal, id, None)

    #
    # Methods to setup and manage skins
    #

    def setupSkins(self, skins):
        """Install or update skins.

        <skins> parameter is a dictionary of {<skin_name>: <skin_path>),}"""
        self.log("Verifying skins")
        skin_installed = 0
        for skin, path in skins.items():
            path = path.replace('/', os.sep)
            self.log(" FS Directory View '%s'" % skin)
            if skin in self.portal.portal_skins.objectIds():
                dv = self.portal.portal_skins[skin]
                oldpath = dv.getDirPath()
                if oldpath == path:
                    self.logOK()
                else:
                    self.log("  Incorrectly installed, correcting path")
                    dv.manage_properties(dirpath=path)
            else:
                skin_installed = 1
                product = self.portal.portal_skins.manage_addProduct['CMFCore']
                product.manage_addDirectoryView(filepath=path, id=skin)
                self.log("  Creating skin")
        if skin_installed:
            all_skins = self.portal.portal_skins.getSkinPaths()
            for skin_name, skin_path in all_skins:
                if skin_name != 'Basic':
                    continue
                path = [x.strip() for x in skin_path.split(',')]
                path = [x for x in path if x not in skins.keys()] # strip all
                if path and path[0] == 'custom':
                    path = path[:1] + [skin for skin in skins.keys()] + path[1:]
                else:
                    path = [skin[0] for skin in skins.keys()] + path
                npath = ', '.join(path)
                self.portal.portal_skins.addSkinSelection(skin_name, npath)
                self.log(" Fixup of skin %s" % skin_name)
            self.flagSkinCacheReset()

    def deleteSkins(self, skinlist):
        self.log('Deleting skins %s' % ', '.join(skinlist))
        for skin in skinlist:
            if skin in self.portal.portal_skins.objectIds():
                self.portal.portal_skins._delObject(skin)
                self.flagSkinCacheReset()

    def flagSkinCacheReset(self):
        self.portal._v_reset_skins = 1

    def setSkinsOnTop(self, skins):
        """Sets the given set of skins at the top of the skin layers, but
        just below the 'custom' layer.
        
        <skins> parameter is a dictionary of {<skin_name>: <skin_path>),}
        """
        skintool = self.portal.portal_skins
        skinSelections = skintool._getSelections()
        for skinName in skins.keys():
            self.log("\nThe layer %s is set at the top of skin :" % skintool.default_skin)
            for skinSelectionName, layers in skinSelections.items():
                layers = layers.split(',')
                layers.insert(1, skinName)
                layers = ','.join(layers)
                skinSelections[skinSelectionName] = layers
                self.log("\n\t--> %s : %s" % (skinSelectionName, layers))
        self.flagSkinCacheReset()
    
    
    def install(self):
        self.log("\nEuralbaPatchFile for Plone : START.")
        self.setupSkins(SKINS)
        self.setSkinsOnTop(SKINS)
        self.finalize()
        self.log("\nEuralbaPatchFile for Plone : END.")



def install(self):
    installer = EuralbaFilePatchInstaller(self)
    installer.install()
    return installer.logResult() 